#ifndef HTTPRI_COMMON_H
#define HTTPRI_COMMON_H

#include <stdio.h>
#include <stdint.h>
#include <stddef.h>

#include <dtsd_ri_plugins.h>

#define _STR(x)      #x
#define _STR2(x)     _STR(x)
#define __SLINE__    _STR2(__LINE__)
#define HERE         __FILE__ ":" __SLINE__ " "

int verbous;
#define dprintf(level, fmt...)									\
	do { if (verbous >= level) fprintf(stderr, fmt); } while(0)
#define dprints(level, str, len)										\
	do { if (verbous >= level) fprints(stderr, str, len); } while (0)

#define SAFE_FREE(ptr)	\
	if( ptr != NULL ){ free(ptr); ptr = NULL; }

#define HTTPRI_DEFAULT_PORT 8080
#define HTTPRI_DEFAULT_BUFSIZE 2048
#define DEFAULT_BACKLOG 500
#define HTTPRI_MAX_TARGETS 64
#define ST_ALIVE 1
#define ST_DEAD 0

#define KEYWORDS_NUM 4
extern const char * keywords[KEYWORDS_NUM];

struct http_request {
	const char *method;
	int method_len;
	const char *uri;
	int uri_len;
	const char *version;
	int version_len;
	const char *user_agent;
	int user_agent_len;
};

struct http_header {
	const char *name;
	int name_len;
	const char *value;
	int value_len;
};

#define HTTPRI_IPADDR_INVALID (0xffffffff)

struct uari_dispatcher {
	const char *name;
	int (*init)(struct ri * ri, uint32_t default_target, const char *ri_optarg);
	uint32_t (*dispatch)(struct ri * ri, const struct http_request *req);
	int (*add_node)(struct ri * ri, uint32_t addr);
	void (*remove_node)(struct ri * ri, uint32_t addr);
	struct node_info * node_infos[KEYWORDS_NUM];
	void (*free_node_info)(struct ri * ri);
};

struct uari_dispatcher_mgr {
	const char *name;
	struct uari_dispatcher *(*create)(const char *name);
};

struct uari_uniq_param {
	struct uari_dispatcher * dispatcher;
	pthread_mutex_t node_list_lock;
	struct server_param * server_param;
	int n_threads;
	struct thread_param * thread_params;
};
	
extern struct uari_dispatcher_mgr rr_dispatcher_mgr;

int resolve_host(const char * hostname, uint32_t * addr);


/* Taken from linux/kernel.h */
/**
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:	the pointer to the member.
 * @type:	the type of the container struct this is embedded in.
 * @member:	the name of the member within the struct.
 *
 */
#define container_of(ptr, type, member) ({			\
	const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)( (char *)__mptr - offsetof(type,member) );})

#endif /* #ifndef HTTPRI_COMMON_H */
