#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <errno.h>
#include <assert.h>
#include <sys/socket.h>

#include "../dtsd/src/dtsd_ri_plugins.h"

static int resolve_host(const char *hostname, uint32_t *addr)
{
	struct hostent *ent = gethostbyname(hostname);
	if (ent == NULL) {
		fprintf(stderr, "Unknown hostname: %s\n", optarg);
		return 1;
	}
	memcpy(addr, ent->h_addr, sizeof(*addr));

	return 0;
}

int main()
{
	int sock;
	int e;
	struct sockaddr_in sin;
	struct ri_intf * ri_intf;
	struct ri * ri;
	uint32_t addr1, addr2;

	sin.sin_family      = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port        = 50001;

	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0)
		return -errno;
	if (bind(sock, (struct sockaddr *) &sin, sizeof(sin)) < 0)
		goto out_close;

	resolve_host("10.0.0.1", &addr1);
	resolve_host("10.0.0.2", &addr2);

	ri_intf = create_ri_intf();
	printf("%s", ri_intf->usage());

	ri = ri_intf->initialize(sock, addr1, 80, "-d rr -c 4");
	assert(ri != NULL);
	ri_intf->add_node(ri, addr1);
	ri_intf->add_node(ri, addr2);
	ri_intf->remove_node(ri, addr1);
	ri_intf->run(ri);

	// TODO send request for proxy

	ri_intf->stop(ri);
	ri_intf->finalize(ri);

	close(sock);
	return 0;

out_close:
	e = errno;
	close(sock);
	return -e;
}
